package de.juplo.facebook.exceptions;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import com.fasterxml.jackson.annotation.JsonRootName;
import org.springframework.security.oauth2.common.exceptions.OAuth2Exception;

/**
 * Base exception for Facebook Graph-Api exceptions.
 * 
 * @author Kai Moritz
 */
@org.codehaus.jackson.map.annotate.JsonDeserialize(using = GraphApiExceptionJackson1Deserializer.class)
@com.fasterxml.jackson.databind.annotation.JsonDeserialize(using = GraphApiExceptionJackson2Deserializer.class)
public class GraphApiException extends OAuth2Exception
{
  private final String type;
  private final int code;

  private int httpErrorCode;


  public GraphApiException(String message, String type, int code)
  {
    super(message);
    this.type = type;
    this.code = code;
  }


  public String getType()
  {
    return type;
  }

  public int getCode()
  {
    return code;
  }

  @Override
  public int getHttpErrorCode()
  {
    return httpErrorCode == 0 ? super.getHttpErrorCode() : httpErrorCode;
  }

  public void setHttpErrorCode(int httpErrorCode)
  {
    this.httpErrorCode = httpErrorCode;
  }

  @Override
  public String toString()
  {
    StringBuilder builder = new StringBuilder();
    builder.append("{error:{\"message\":\"");
    builder.append(getMessage().replaceAll("\"", "\\\""));
    builder.append("\",\"type\":");
    builder.append(type.replaceAll("\"", "\\\""));
    builder.append("\",\"code\":");
    builder.append(code);
    builder.append("}}");
    return builder.toString();
  }


  /**
   * This class represents an error message from the Graph-API
   *
   * @see https://developers.facebook.com/docs/graph-api/using-graph-api/v2.5#errors
   */
  @JsonRootName("error")
  @JsonPropertyOrder({ "message", "type", "code", "error_subcode", "error_user_title", "error_user_msg", "fbtrace_id" })
  public static class FacebookErrorMessage
  {
    @JsonProperty("message")
    String message;
    @JsonProperty("type")
    String type;
    @JsonProperty("code")
    Integer code;
    @JsonProperty("error_subcode")
    Integer subCode;
    @JsonProperty("error_user_title")
    String userTitle;
    @JsonProperty("error_user_msg")
    String userMessage;
    @JsonProperty("fbtrace_id")
    String traceId;
  }
}
